'calculations follow those in Baltagi 2008  "Econometric Analysis of Panel Data", page 194.
!debug = 1
if !debug = 0 then
	logmode +all
endif

'check workfile is panel
if @ispanel=0 then
	@uiprompt("Procedure can only be run in panel workfiles")
	stop
endif

'check that an object exists
%type = @getthistype
if %type="NONE" then
	@uiprompt("No object found, please open an Equation object")
	stop
endif
'check that _this object is an equation
if %type<>"EQUATION" then
	@uiprompt("Procedure can only be run from an Equation object")
	stop
endif

'check that equation was estimated by LS, and by list.
%method = _this.@method
if %method <> "LS" then
	@uiprompt("Procedure can only be run from Equations estimated by Least Squares")
	stop
endif
if _this.@bylist = 0 then
	@uiprompt("Procedure can only be run from Equations estimated by list")
	stop
endif

'check equation was estimated with no effects
%ops = _this.@options
if @instr(%ops, "CX=") > 0 then
	@uiprompt("Procedure can only be run from Equations with no cross-section effects")
	stop
endif

!dontBP = 0
!dontHonda = 0
!dontSLM = 0
!dontKW = 0
!dontGHM = 0
if @len(@option(1)) then
	!dontHonda = hasoption("noHonda")
	!dontBP = hasoption("noBP")
	!dontSLM = hasoption("noSLM")
	!dontKW = @hasoption("noKW")
	!dontGHM = @hasoption("noGHM")
else
	!result = @uidialog("caption","Inclusion and Exclusion of tests", _
	"text","Exclude: ", _
	"check",!dontBP,"Breusch-Pagan Test", _
	"check",!dontSLM,"Standard Lagrange multiplier Test", _
	"check",!dontKW,"King-Wu Test", _
	"check",!dontGHM,"Gourieroux-Holly-Monfort Test")
	if !result = -1 then
		stop
	endif
endif
if !dontBP + !dontSLM + !dontKW + !dontGHM = 4 then
	@uiprompt("Please select at least one of tests.")
	stop
endif
%smpl = @pagesmpl
smpl @all

!n = _this.@regobs

'make residual series from equation
%resn = @getnextname("_res")
_this.makeresids {%resn}

'make squared residuals
%res2n = @getnextname("_ressq")
series {%res2n}={%resn}*{%resn}
!sum_resid_sq = @sum({%res2n})

%tempn = @getnextname("_tempn")	'corss-sectional
series {%tempn} = @crossid
%tempt = @getnextname("_tempt")	'period
series {%tempt} = @trend

'calculate N and T
!n_t = @max({%tempn})	
!capT = @max({%tempt})+1
 
%eye = @getnextname("_eye")
matrix {%eye} = @identity(!n_t)

%d1 = @getnextname("_d1")
%d2 = @getnextname("_d2")
matrix(!n,!n_t) {%d1}
matrix(!n,!capT) {%d2}
if !dontSLM = 1 then
	d(noerr) {%d1} {%d2} 
endif

'Find sum of time sums squared, and sum of N squared
!sum_sq_time_sums = 0
!m22 = 0

for !i=0 to !capT-1
	smpl if @trend=!i
	!obst = @obs({%resn})
	if !obst>0 then 
		!sumt = @sum({%resn})
		!sum_sq_time_sums=!sum_sq_time_sums+!sumt^2
		!m22 = !m22 + !obst^2
	endif
next

'Find sum of cross section sums squared, and sum of T squared
!sum_sq_sums=0
!m11=0
!unbalanced = 0
for !i=1 to @max({%tempn})
	smpl if @crossid=!i
	!obs = @obs({%resn})
	if !obs <> !capT then
		!unbalanced = 1		
	endif
	if !obs>0 then 'i.e if cross-section was used in the equation.
		!sum=@sum({%resn})
		!sum_sq_sums=!sum_sq_sums+!sum^2
		!m11 = !m11 + !obs^2
	endif
next

if !dontSLM = 0 then
	!row = 1
	if !unbalanced = 0 then 'balanced panel
		matrix {%d1} = @kronecker(@identity(@max({%tempn})),@ones(@max({%tempt})+1,1))
		matrix {%d2} = @kronecker(@ones(@max({%tempn}),1),@identity(@max({%tempt})+1))
		if @errorcount > 0 then
			d(noerr) {%d1} {%d2}
			stop
		endif
	else 'unbalanced panel
		for !i=0 to !capT-1
			smpl if @trend=!i
			!obst = @obs({%resn})
			if !obst > 0 then 
				%d1t = @getnextname("_d1t")
				%d2t = @getnextname("_d2t")
				matrix(!obst,!n_t) {%d1t}
				matrix(!obst,1) {%d2t}			
				%ti = @getnextname("_ti")
				vector {%ti} = {%tempn}	
				%eyet = @getnextname("_eyet")
				for !ii=1 to !obst
					rowvector {%eyet} = @rowextract({%eye},{%ti}(!ii))
					rowplace({%d1t},{%eyet},!ii)
				next
				matrix {%d2t} = {%d1t}*@ones(!n_t,1)
				matplace({%d1}, {%d1t}, !row,1)
				matplace({%d2}, {%d2t}, !row,!i+1) 'diagonal
				!row = !row + !obst				
				d(noerr) {%ti} {%d1t} {%d2t} {%eyet}
			endif
		next
	endif
endif
d(noerr) {%eye} {%tempn} {%tempt} {%resn} {%res2n}
smpl {%smpl}
 
'Breusch-Pagan (1980) --------------------------------
'H0: sigma_mu=0
!A1	=	(!sum_sq_sums/!sum_resid_sq)-1
!BP1	=	(1/2)*!n^2/(!m11-!n) * (!A1)^2
!BPprob1 = 1-@cchisq(!BP1,1)	'calculate p-value
'H0: sigma_lambda=0
!A2	=	(!sum_sq_time_sums/!sum_resid_sq)-1
!BP2	=	(1/2)*!n^2/(!m22-!n) * (!A2)^2
!BPprob2 = 1-@cchisq(!BP2,1)	'calculate p-value
'H0: sigma_mu = sigma_lambda=0
!BP	=	!BP1 + !BP2
!BPprob = 1-@cchisq(!BP,1)	'calculate p-value

'Calculation for one-sided statistics
!LM1 = !n*!A1/(@sqrt(2*(!m11-!n))) '@sqrt(BP1)
!LM2 = !n*!A2/(@sqrt(2*(!m22-!n)))

if !dontHonda = 0 then
	'Honda (1985) ---------------------------------------------
	'H0: sigma_mu=0
	!Honda1 = !LM1
	!hondaprob1 = 1-@cnorm(!honda1)
	'H0: sigma_lambda=0
	!Honda2 = !LM2
	!hondaprob2 = 1-@cnorm(!honda2)
	'H0: sigma_mu = sigma_lambda=0, note: 'Honda "handy" one-sided 
	!Honda = (!LM1+!LM2)/@sqrt(2)	
	!hondaprob = 1-@cnorm(!honda)		
endif
	 
if !dontSLM = 0 then
	'Standard Lagrange Multiplier (SLM) ------------
	%vars = _this.@varlist
	!K = @wcount(%vars)-1
	!p = !n-(!K)
	
	%grp = @getnextname("_grp")
	%reg= @wmid(%vars,2)
	group {%grp} {%reg}
	%z = @getnextname("_z")
	matrix {%z} = @convert({%grp})
	d(noerr)  {%grp} 
	%pz = @getnextname("_pz")
	matrix {%pz} = @identity(!n) - {%z}*((_this.@coefcov)/(_this.@se*_this.@se))*@transpose({%z})
	
	%d11 = @getnextname("_d11")
	%d22 = @getnextname("_d22")
	matrix {%d11} = {%d1}*@transpose({%d1})
	matrix {%d22} = {%d2}*@transpose({%d2})
	
	!tr11 = @trace({%d11}*{%pz})
	!tr22 = @trace({%d22}*{%pz})
	!tr_sq_11 = @trace({%d11}*{%pz}*{%d11}*{%pz})
	!tr_sq_22 = @trace({%d22}*{%pz}*{%d22}*{%pz})	
	
	!eLM1 = !tr11/!p
	!eLM2 = !tr22/!p
	
	!vLM1 = (2/(!p^2*(!p+2)))*(!p*!tr_sq_11 - !tr11^2) 
	!vLM2 = (2/(!p^2*(!p+2)))*(!p*!tr_sq_22 - !tr22^2)
	
	'H0: sigma_mu=0
	!SLM1 = ((!A1+1) - !eLM1) /@sqrt(!vLM1)
	!SLM1prob = 1-@cnorm(!SLM1)
	'H0: sigma_lambda=0
	!SLM2 = ((!A2+1) - !eLM2) /@sqrt(!vLM2)
	!SLM2prob = 1-@cnorm(!SLM2)
	
	d(noerr) {%z} {%pz} {%d1} {%d2} {%d11} {%d22}
endif

if !dontKW = 0 then
	'King-Wu (1997) ------------------------------------------
	'H0: sigma_mu=0
	!KW1 = !LM1
	!KW1prob = 1-@cnorm(!KW1)
	'H0: sigma_lambda=0
	!KW2 =  !LM2
	!KW2prob = 1-@cnorm(!KW2)
	'H0: sigma_mu = sigma_lambda=0
	!KW = (@sqrt(!m11-!n)/@sqrt(!m11+!m22-2*!n))*!LM1 + (@sqrt(!m22-!n)/@sqrt(!m11+!m22-2*!n))*!LM2
	!KWprob = 1-@cnorm(!KW)	
endif

if !dontGHM = 0 then
	'Gourierous, Holly and Monfort (1982) -----------
	if !LM1 > 0 and !LM2 > 0 then
		!GHM = !LM1^2 + !LM2^2
	endif
	if !LM1 > 0 and !LM2 <= 0 then
		!GHM = !LM1^2
	endif
	if !LM1 <= 0 and !LM2 > 0 then
		!GHM = !LM2^2
	endif
	if !LM1 <= 0 and !LM2 <= 0 then
		!GHM = 0
	endif
	!GHMprob = (1/2)*(1-@cchisq(!GHM,1)) + (1/4)*(1-@cchisq(!GHM,2)) 	
endif
!estobs = _this.@regobs

'Results
%tn = @getnextname("_table")
Table(30,4) {%tn}
{%tn}.setwidth(1) 18
{%tn}.setwidth(2:4) 12
{%tn}.setjust(a1:a30) left
!rowcounter=1
{%tn}(!rowcounter,1) = "Lagrange multiplier (LM) test for panel data"
!rowcounter = !rowcounter + 1
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MM")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1
%samplestr = "Sample: " + %smpl
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%obsstr = "Total panel observations: " + @str(!estobs)
{%tn}(!rowcounter,1) = %obsstr
!rowcounter=!rowcounter+1
%probin = "Probability in ()"
{%tn}(!rowcounter,1) = %probin
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1
'---------------------------
{%tn}(!rowcounter,1) = "Null (no rand. effect)"
{%tn}(!rowcounter,2) = "Cross-section"
{%tn}(!rowcounter,3) = "Period"
{%tn}(!rowcounter,4) = "Both"
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Alternative"
{%tn}(!rowcounter,2) = "One-sided"
{%tn}(!rowcounter,3) = "One-sided"
{%tn}(!rowcounter,4) = ""
!rowcounter = !rowcounter + 1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1
if !dontBP = 0 then
	{%tn}(!rowcounter,1) = "Breusch-Pagan"
	{%tn}(!rowcounter,2)=!BP1
	{%tn}(!rowcounter,3) = !BP2
	{%tn}(!rowcounter,4) = !BP
	!rowcounter=!rowcounter+1
	{%tn}(!rowcounter,2) = "(" + @str(!BPprob1,"f.4") + ")"
	{%tn}(!rowcounter,3) = "(" + @str(!BPprob2,"f.4") + ")"
	{%tn}(!rowcounter,4) = "(" + @str(!BPprob,"f.4") + ")"
	{%tn}.setformat(!rowcounter,2,!rowcounter,4) f.4
	!rowcounter=!rowcounter+1	
endif
if !dontHonda = 0 then
	{%tn}(!rowcounter,1) = "Honda"
	{%tn}(!rowcounter,2) = !Honda1
	{%tn}(!rowcounter,3) = !Honda2
	{%tn}(!rowcounter,4) = !Honda
	!rowcounter=!rowcounter+1
	{%tn}(!rowcounter,2) = "(" + @str(!hondaprob1,"f.4") + ")"
	{%tn}(!rowcounter,3) = "(" + @str(!hondaprob2,"f.4") + ")"
	{%tn}(!rowcounter,4) = "(" + @str(!hondaprob,"f.4") + ")"
	{%tn}.setformat(!rowcounter,2,!rowcounter,4) f.4
	!rowcounter=!rowcounter+1	
endif
if !dontKW = 0 then
	{%tn}(!rowcounter,1) = "King-Wu"
	{%tn}(!rowcounter,2) = !KW1
	{%tn}(!rowcounter,3) = !KW2
	{%tn}(!rowcounter,4) = !KW
	!rowcounter=!rowcounter+1
	{%tn}(!rowcounter,2) = "(" + @str(!KW1prob,"f.4") + ")"
	{%tn}(!rowcounter,3) = "(" + @str(!KW2prob,"f.4") + ")"
	{%tn}(!rowcounter,4) = "(" + @str(!KWprob,"f.4") + ")"
	{%tn}.setformat(!rowcounter,2,!rowcounter,4) f.4
	!rowcounter=!rowcounter+1	
endif		 
if !dontSLM = 0 then
	{%tn}(!rowcounter,1) = "SLM"
	{%tn}(!rowcounter,2) = !SLM1
	{%tn}(!rowcounter,3) = !SLM2
	{%tn}(!rowcounter,4) = "--"
	!rowcounter=!rowcounter+1	
	{%tn}(!rowcounter,2) = "(" + @str(!SLM1prob,"f.4") + ")"
	{%tn}(!rowcounter,3) = "(" + @str(!SLM2prob,"f.4") + ")"
	{%tn}(!rowcounter,4) = "--"
	{%tn}.setformat(!rowcounter,2,!rowcounter,4) f.4
	!rowcounter=!rowcounter+1
endif
if !dontGHM = 0 then
	{%tn}(!rowcounter,1) = "GHM"
	{%tn}(!rowcounter,2) = "--"
	{%tn}(!rowcounter,3) = "--"
	{%tn}(!rowcounter,4) = !GHM
	!rowcounter=!rowcounter+1
	{%tn}(!rowcounter,2) = "--"
	{%tn}(!rowcounter,3) = "--"
	{%tn}(!rowcounter,4) = "(" + @str(!GHMprob,"f.4") + ")"
	{%tn}.setformat(!rowcounter,2,!rowcounter,4) f.4
	!rowcounter=!rowcounter+1	
endif
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

_this.display {%tn}
d {%tn}
